from velait.velait_fastapi.api.pagination.schemas import Page
from velait.velait_fastapi.api.pagination.pagination import paginate
from velait.common.services.pagination import get_page_limits, get_offset_limits
